/**************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
* misc stuff
*
**************************************************************************/
#include "defs.h"
#include "lib.p"

PUBLIC int       iptrk_fd = -1;
PUBLIC time_t    timer, old_watch_time;
PUBLIC struct tm *time_st, old_time_st;

/**************************************************************************
*
**************************************************************************/
PUBLIC char *parse_cmdline(int argc, char *argv[], int i)
{
  i++;
  if(i >= argc)
  {
    printf("Error: missing -f parameter.\n");
    exit(-1);
  }

  return argv[i];
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void welcome(void)
{
  printf(WELCOME "\n");
}

/**************************************************************************
*
**************************************************************************/
PUBLIC time_t datestr2time(char *s)
{
  char      str[STR_LEN + 1], c = 0;
  time_t    tm;
  struct tm t, *tl;
  int       i;

  memset(&t, 0, sizeof(struct tm));
  for(i = 0; i < strlen(s); i++)
    if(isdigit(s[i]))
      str[i] = s[i];
    else
    {
      str[i] = ' ';
      c++;
    }
  str[i] = 0;

  s = strchr(str, ' ');
  if(s == NULL)
    return -1;

  if((int)(s - str) == 4 && c == 4)
  {
    if(sscanf(str, "%d %d %d %d %d",
      &(t.tm_year), &(t.tm_mon), &(t.tm_mday), 
      &(t.tm_hour), &(t.tm_min)) != 5)
    {
      return -1;
    }

    t.tm_mon--;
    t.tm_year = t.tm_year - 1900;
    t.tm_isdst = -1;
    return mktime(&t);
  }

  if(c != 1)
    return -1;

  tm = time(NULL);
  tl = localtime(&tm);
  memcpy(&t, tl, sizeof(struct tm));

  if(sscanf(str, "%d %d", &(t.tm_hour), &(t.tm_min)) != 2)
    return -1;

  return mktime(&t);
}

PUBLIC void format_date_entry(time_t t, char *s)
{ 
  struct tm *tp, ts;
    
  tp = localtime(&t);
  if(tp == NULL)
    tp = &ts;
  sprintf(s, "%04d:%02d:%02d:%02d:%02d",
    tp->tm_year + 1900, tp->tm_mon + 1, tp->tm_mday, tp->tm_hour, tp->tm_min);
}

/**************************************************************************
* odds and ends
**************************************************************************/
PUBLIC int ignore_space(char **s)
{
  for(;; (*s)++)
  {
    if(**s == 0)
      return 0;
    if(!isspace(**s))
      return 1;
  }

  return 0;
}

PUBLIC int ignore_field(char **s)
{
  for(;; (*s)++)
  {
    if(**s == 0)
      return 0;
    if(isspace(**s))
      return 1;
  }

  return 0;
}

#define MAX_VOL_DIGITS 60
PUBLIC int get_next_vol_u64(char **buf, U_64 *val)
{
  int  l = 0, err = 0;
  U_64 e = 1;

  void w(char *s)
  {
    if(*s == 0 || isspace(*s) || l > MAX_VOL_DIGITS)
    {
      *buf = s;
      return;
    }

    l++;
    w(s + 1);

    if(isdigit(*s))
    {
      *val += (U_64)(*s - '0') * e;
      e *= 10;
    }
    else if(*s != ',')
      err = 1;
  }

  *val = 0;
  if(!ignore_space(buf))
    return 0;
  w(*buf);
  if(err)
    return 0;

  return 1;
}

PUBLIC void print_vol_u64(char *buf, U_64 vol)
{
  int o = 0;

  void p(U_64 v)
  {
    if(v >= 1000)
      p(v / 1000);

    if(v >= 1000)
      sprintf(buf + o, ",%03llu", v % 1000);
    else
      sprintf(buf + o, "%llu", v % 1000);
    o = strlen(buf);
  }

  p(vol);
}

PUBLIC void str_hh_mm(time_t t, char *s)
{
  if(t < 0)
    t = 0;
  if((t % 60) < 30)
    t /= 60;
  else
  {
    t /= 60;
    t++;
  }
  sprintf(s, "%d:%02d", (int)(t / 60), (int)(t % 60));
}

PUBLIC void str_date(time_t t, char *s)
{
  struct tm *tm_st;
  static char *mon_tab[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun",
                            "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

  tm_st = localtime((time_t *)&t);
  if(tm_st != NULL)
  {
    sprintf(s, "%s%02d %02d:%02d:%02d",
      mon_tab[tm_st->tm_mon], tm_st->tm_mday,
      tm_st->tm_hour, tm_st->tm_min, tm_st->tm_sec);
    return;
  }

  strcpy(s, "????? ??:??:??");
}

PUBLIC inline void file_date(struct tm *ts, char *s)
{
  sprintf(s, "%04d.%02d.%02d.%02d.%02d",
    1900 + ts->tm_year,
    ts->tm_mon + 1,
    ts->tm_mday,
    ts->tm_hour, 
    ts->tm_min);
}

PUBLIC inline void strncpy2(char *dst, char *src, int len)
{
  strncpy(dst, src, len);
  dst[len] = 0;
}

PUBLIC int day2i(char *str)
{
  static char *dt_table[] = {"sun", "mon", "tue", "wed", "thu", "fri", "sat"};
  int i;

  for(i = 0; i < 7; i++)
    if(!strcasecmp(str, dt_table[i]))
      return i;

  return -1;
}

PUBLIC int digit2i(char c)
{
  if(c == ' ')
    return 0;

  if(c < '0' || c > '9')
    return 0;

  return '0' - c;
}

PUBLIC void strip_cr(char *str)
{
  int s;

  s = strlen(str);
  if(s > 0)
    if(str[s - 1] == '\n')
      str[s - 1] = 0;
}

/**************************************************************************
*
**************************************************************************/
PUBLIC int tally_processes(char *process_name)
{
  DIR *dp;
  struct dirent *dir_entry;
  int r = 0;

  dp = opendir("/proc");
  if(dp == NULL)
    return -1;

  while((dir_entry = readdir(dp)) != NULL)
  {
    FILE *fp;
    char str[STR_LEN + 1], substr[STR_LEN + 1];
    int e_pid;

    if(!sscanf(dir_entry->d_name, "%d", &e_pid))
      continue;

    strcpy(str, "/proc/");
    strcat(str, dir_entry->d_name);
    strcat(str, "/stat");
    fp = fopen(str, "r");
    if(fp == NULL)
      continue;

    fgets(str, STR_LEN, fp);
    fclose(fp);
    if(sscanf(str, "%*u %s", substr))
      if(!strcmp(substr, process_name))
        r++;
  }
  closedir(dp);

  return r;
}

/**************************************************************************
* promiscuous mode
**************************************************************************/
PUBLIC void promiscuous_mode(int flag)
{
  int          nif_sock, i;
  struct ifreq ifr;
  char         str[STR_LEN];

  log_m('I', NULL, " opening device socket");
  nif_sock = socket(PF_PACKET, SOCK_PACKET, htons(ETH_P_ALL));
  if(nif_sock < 0)
  {
    log_e(__func__, "::socket()");
    exit(-1);
  }

  for(i = 0; i < BOOT_T_NIFS; i++)
  {
    if(boot.nifs[i][0] == 0)
      break;

    sprintf(str, " setting %s promiscuous mode %s",
      boot.nifs[i], flag == 1 ? "on" : "off");
    log_m('I', NULL, str);

    memset(&ifr, 0, sizeof(ifr));
    strcpy(ifr.ifr_name, boot.nifs[i]);
    if(ioctl(nif_sock, SIOCGIFFLAGS, &ifr) < 0)
    {
      log_e(__func__, "::ioctl(SIOCGIFFLAGS)");
      exit(-1);
    }

    if(flag)
      ifr.ifr_flags |= IFF_PROMISC;
    else
      ifr.ifr_flags &= ~IFF_PROMISC;

    if(ioctl(nif_sock, SIOCSIFFLAGS, &ifr) < 0)
    {
      log_e(__func__, "::ioctl(SIOCSIFFLAGS)");
      exit(-1);
    }
  }

  log_m('I', NULL, " closing device socket");
  close(nif_sock);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void *alloc_mem(int mem_size)
{
  void *r;

  r = malloc(mem_size);
  if(r == NULL)
  {
    log_e(__func__, "::malloc()");
    exit(-1);
  }
  return r;
}

PUBLIC void *realloc_mem(void *ptr, int mem_size)
{
  void *r;

  r = realloc(ptr, mem_size);
  if(r == NULL)
  {
    log_e(__func__, "::realloc()");
    exit(-1);
  }
  return r;
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void email_operator(char *subj, char *msg)
{
  int       r;
  char      str[STR_LEN + 1],
            date_stamp[STR_LEN + 1];
  struct tm *tm;

  if(boot.email[0][0] == 0)   /*if no email entries, then return*/
    return;

  tm = localtime((const time_t *)&timer);
  if(tm == NULL)
    return;
  if(strftime(date_stamp, STR_LEN, "%a, %d %b %Y %H:%M:%S %z (%Z)", tm) == 0)
    return;

  r = fork();
  if(r == -1)
  {
    log_e(__func__, "::fork()");
    exit(-1);
  }
  else
  {
   /*
    * child
    */
    if(r == 0)
    {
      FILE *fp;
      int i;

      for(i = 0; i < BOOT_T_EMAIL_LIM; i++)
      {
        if(boot.email[i][0] == 0)
          break;

        sprintf(str, "%s %s", SENDMAIL_PATH, boot.email[i]);
        fp = popen(str, "w");
        if(fp != NULL)
        {
          fprintf(fp,
            "To: %s <Operator>\n"
            "From: root@" HOSTNAME " <iptrk>\n"
            "Date: %s\n"
            "Subject: %s\n\n"
            "%s\n"
            "--\niptrk R" REVISION " (" HOSTNAME ")\n",
            boot.email[i],
            date_stamp, subj, msg);

          pclose(fp);
        }
      }
      exit(0);
    }

   /*
    * parent
    */
    sprintf(str, " emailing users (created child %d)", r);
    log_m('I', NULL, str);
  }
}

/**************************************************************************
*
**************************************************************************/
PUBLIC void wait_forever(void)
{
  for(;;);
}

PUBLIC void attach_tty(char *name)
{
  char str[STR_LEN + 1];
  int  fd, i;

  sprintf(str, "/dev/%s", name);
  if(chown(str, 0, 0) == -1)
  {
    log_e(__func__, "::chown()");
    wait_forever();
  }
  if(chmod(str, 0600))
  {
    log_e(__func__, "::chmod()");
    wait_forever();
  }

  signal(SIGHUP, SIG_IGN);
  fd = open(str, O_RDWR, 0);
  if(fd == -1)
  {
    log_e(__func__, "::open(/dev/tty)");
    wait_forever();
  }

  if(ioctl(fd, TIOCSCTTY, (void *)1) == -1)
  {
    log_e(__func__, "::ioctl(TIOCSCTTY)");
    wait_forever();
  }

  if(!isatty(fd))
  {
    log_m('E', __func__, "::isatty()- defined terminal not a tty");
    wait_forever();
  }

  vhangup();
  close(2);
  close(1);
  close(0);
  close(fd);

  fd = open(str, O_RDWR, 0);
  if(fd == -1)
  {
    log_e(__func__, "::open2(/dev/tty)");
    wait_forever();
  }

  for(i = 0; i < 1; i++)
    if(dup(0) == -1)
    {
      log_e(__func__, "::dup()");
      wait_forever();
    }

  signal(SIGHUP, SIG_DFL);
}

/**************************************************************************
*
**************************************************************************/
PUBLIC int log_ip_fast_find(FILE *fp, time_t stime)
{
  long   size,
         size_1,
         size_2,
         size_3,
         size_4,
         last_pos = 0;
  time_t t = 0;
  char   str[STR_LEN + 1];

  int find(long s, long e)
  {
    fseek(fp, s, SEEK_SET);
    for(;;)
    {
      if(feof(fp))
        break;

      if(ftell(fp) >= e)
        break;

      if(fgets(str, STR_LEN, fp) == NULL)
        break;

      if(str[0] == 'o')
        if(sscanf(str + 2, "%lx", &t) == 1)
          return 1;
    }
    return 0;
  }

 /*
  * start
  */
  fseek(fp, 0, SEEK_END);
  size = ftell(fp);
  if(size < 8192)
  {
    if(!find(0, size))
      return 0;
    fseek(fp, last_pos, SEEK_SET);
    return 1;
  }

  size_4 = size - (size >> 4);
  size_3 = size - (size >> 3);
  size_2 = size - (size >> 2);
  size_1 = size - (size >> 1);
  if(!find(size_4, size))
    if(!find(size_3, size))
      if(!find(size_2, size_4))
        if(!find(size_1, size_3))
          if(!find(0, size_2))
            return 0;
  fseek(fp, last_pos, SEEK_SET);
  return 1;
}
